/*
 * Copyright (c) 2010 Maxim Sinev
 * Licensed under GNU General Public License V2
 */

package naca4builder;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;

/**
 *
 * @author max
 */

public class Main implements Runnable {

    public static void main(String[] args) throws IOException {
        if(args.length!=4  || args[0].length()!=4) {
            System.out.println("Arguments: <AIRFOIL CODE> <LENGTH> <OFFSET> <DPI> ");
            System.out.println("Example: 2312 1000 -4 100");
            return;
            }
        String code=args[0];
        Main run=new Main(Integer.parseInt(code.substring(0,1)),
                Integer.parseInt(code.substring(1,2)),
                Integer.parseInt(code.substring(2,4)),
                Integer.parseInt(args[1]),Integer.parseInt(args[2]));
//        Main run=new Main(3,4,14,1000,-4); //  3_3_14(foil) 1000mm(hord) -4mm(offset)
//        Main run=new Main(0,0,14,1000);
        run.initGraphics(Integer.parseInt(args[3])); // 100dpi
        run.run();
       }


    /**
     * @param args the command line arguments
     */
    final int im_;  // NACA mptt
    final int ip_;
    final int tt_;

    final double t; // Thickness
    final double m; // Camber maximum
    final double p; // Camber maximum location
    final double step=0.0001;
    final boolean chambered;

    final long hord;
    final int    ioffset;
    final double offset;
    /**
     * Units of hord to inch 25.4 for millimeters
     */
    final double scale=25.4;

    NACAPoint pBase=new NACAbase(this);
    NACAPoint pCamber=new NACAcamberline(pBase, this);
    NACAPoint pLower=new NACAlower(pCamber, this);
    NACAPoint pUpper=new NACAupper(pCamber, this);
    NACAoffset pLowerShift=null;
    NACAoffset pUpperShift=null;
    
    public Main(int m, int p, int tt, int hord) {
       im_=m;
       ip_=p;
       tt_=tt;
       this.t=tt/100.;
       this.p=p/10.;
       this.m=m/100.;
       this.hord=hord;
       offset=0;
       ioffset=0;
       chambered=(m!=0 || p!=0);
       }

    public Main(int m, int p, int tt, int hord, double offset) {
       im_=m;
       ip_=p;
       tt_=tt;
       this.t=tt/100.;
       this.p=p/10.;
       this.m=m/100.;
       this.hord=hord;
       chambered=(m!=0 || p!=0);
       ioffset=(int) offset;
       this.offset=offset/hord;
       pLowerShift=new NACAoffset(-this.offset, pLower);
       pUpperShift=new NACAoffset(this.offset, pUpper);
       }


    BufferedImage im;
    WritableRaster raster;
    int linestep=20;
    int topshift=linestep*5;

    long dpi;
    int c;
    int iheight;
    int iwidth;
    int imidle;

    public void initGraphics(int dpi) {
       this.dpi=dpi;
       c=(int) (hord/scale * dpi);
       iheight=c+10+topshift;
       iwidth=(int) ((t * c )*2 + 10)/2;
       imidle=iwidth/2;
       iwidth+=iwidth*0.5;
       byte r[]={0,-1,-1,0};
       byte g[]={0,-1,0,-1};
       byte b[]={0,-1,0,0};
        im = new BufferedImage(iwidth, iheight, 
                BufferedImage.TYPE_BYTE_BINARY, new IndexColorModel(2,4,r,g,b));
        raster=im.getRaster();

        for(int y=0; y<iwidth; y++) {
          for(int x=0; x<iheight; x++) {
            raster.setSample(y, x, 0, 1);
            }
          }
    }

    public String nacaID() {
      return  Integer.toString(im_)+
              Integer.toString(ip_)+
              ((tt_<10)?"0":"")+
              Integer.toString(tt_);
      }

    void draw(NACAPoint p, int color) {
      int iy=(int) Math.round(p.y*c);
      int ix=(int) Math.round(p.x*c);
      raster.setSample(imidle+iy, ix+topshift, 0, color);
      }

    @Override
    public void run() {
            while (pBase.x < 1) {
             pBase.calc();


             int iy=(int) Math.round(pBase.y*c);
             int ix=(int) Math.round(pBase.x*c);
//             raster.setSample(imidle+iy, ix+topshift, 0, 3);
//             raster.setSample(imidle-iy, ix+topshift, 0, 3);

             draw(pCamber,2);
             draw(pUpper,2);
             draw(pLower,2);
             if(pUpperShift!=null) draw(pUpperShift,0);
             if(pLowerShift!=null) draw(pLowerShift,0);

             raster.setSample(imidle, ix+topshift, 0, 3);
  //           raster.setSample(iwidth-1, ix+topshift, 0, 0);
            }
            Graphics g = im.getGraphics();
            g.setColor(Color.BLACK);
            int liney = 0;
            g.drawString("NACA " + nacaID(), 10, liney += linestep);
            g.drawString("dpi=" + dpi, 10, liney += linestep);
            g.drawString("hord=" + hord+"mm", 10, liney += linestep);
            g.drawString("Lfoil=" + Math.round((pUpper.l+pLower.l)*hord)+"mm", 10, liney += linestep);
            g.drawString("offset=" + ioffset+"mm", 10, liney += linestep);
            g.drawString("S=" + ((pBase.s) / (pBase.maxy)), 10, liney += linestep);
            g.drawString("H=" + (pBase.maxy * 2), 10, liney += linestep);
        try {
            File outf=new File("naca"+nacaID()+"-hord"+hord+"mm-at"+dpi+"dpi"+((ioffset==0)?"":("-f"+ioffset))+".png");
            ImageIO.write(im, "PNG", outf);
            System.out.println(outf.getAbsolutePath());
        } catch (IOException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }

    }

        // TODO code application logic here
    }


